#!/usr/bin/env python3

import re
import click

from cdbApi import SimpleLocationInformation
from cdbApi import LogEntryEditInformation
from cdbApi import ItemStatusBasicObject
from cdbApi import ApiException

from CdbApiFactory import CdbApiFactory

from cdbCli.common.cli.cliBase import CliBase


class AliasedGroup(click.Group):

    def get_command(self, ctx, cmd_name):
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx)
                   if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail('Too many matches: %s' % ', '.join(sorted(matches)))


@click.command(cls=AliasedGroup)
def main():
    pass


def set_cdb_dist(dist):
    """ Sets the address for the CDB distribution. Options are the sandbox, development, and production.

        :param dist: the cdb distribution
        :return cli: the CliBase object for the chose cdb distribution
    """

    cli = CliBase()

    if dist == "sandbox":
        cli.api_factory = CdbApiFactory("http://tinkerbox.aps.anl.gov:8080/cdb")
    elif dist == "dev":
        cli.api_factory = CdbApiFactory("https://cdb-dev.aps.anl.gov/cdb_dev")
    elif dist == "prod":
        cli.api_factory = CdbApiFactory("https://cdb.aps.anl.gov/cdb")
    elif dist == "local":
        cli.api_factory = CdbApiFactory("http://192.168.1.201:8080/cdb")

    return cli


#############################################################################################
#                                                                                           #
#                        Get info about catalog items from name                             #
#                                                                                           #
#############################################################################################
def get_inventory_info_by_catalog_id(catalog_id, field, cli):
    """
    This function prints info about a given catalog item's associated inventory items.

        :param catalog_id: ID of the catalog item
        :param field: The name of the info field requested
        :param cli: The necessary CliBase object
    """

    factory = cli.require_api()
    item_api = factory.getItemApi()

    derived_items = item_api.get_items_derived_from_item_by_item_id(catalog_id)
    fields_requested = field.split(",")

    for item in derived_items:

        info_list = []
        for f in fields_requested:
            f = f.strip(" ")

            # Get info for inventory items
            if f == "status":
                status = (item_api.get_item_status(item.to_dict()['id'])).to_dict()['value']
                info = str(status)
            if f == "location":
                location = (item_api.get_item_location(item.to_dict()['id'])).to_dict()['location_string']
                info = str(location)
            if f == "serial number":
                info = item.to_dict()['item_identifier1']
            else:
                try:
                    info = str(item.to_dict()[f])
                except KeyError:
                    info = "Invalid field"

            info_list.append(info)

        # Print info to user

        print_str = "\t" + item.to_dict()['name']

        for inf in info_list:
            if inf is None:
                inf = "None"
            print_str += " : " + inf

        try:
            click.echo(print_str)

        except KeyError:
            click.echo("Invalid field")
            break


def get_specific_inventory_info_by_catalog_id(catalog_id, field, inventory, cli):
    """
    This function prints info about a given catalog item's associated inventory items.

        :param catalog_id: ID of the catalog item
        :param field: The name of the info field requested
        :param inventory: The specific inventory names that are needed
        :param cli: The necessary CliBase object
    """

    factory = cli.require_api()
    item_api = factory.getItemApi()

    derived_items = item_api.get_items_derived_from_item_by_item_id(catalog_id)

    found = False

    # expression must be at beginning of name
    r = "^" + inventory.lower() + "$"

    # change SQL wildcard '?' to regex wildcard '.'
    if '?' in inventory:
        r = r.replace('?', '.')

    # change SQL wildcard '*' to regex wildcard '.*'
    if '*' in inventory:
        r = r.replace('*', '.*')

    fields_requested = field.split(",")

    for i in range(len(derived_items)):

        matches = re.findall(r, (derived_items[i].to_dict()['name']).lower())

        if matches:
            info_list = []
            for f in fields_requested:
                f = f.strip(" ").lower()

                # Get info to print
                if f == "status":
                    status = (item_api.get_item_status(derived_items[i].to_dict()['id'])).to_dict()['value']
                    info = str(status)
                if f == "location":
                    location = (item_api.get_item_location(derived_items[i].to_dict()['id'])).to_dict()['location_string']
                    info = str(location)
                if f == "serial number":
                    info = derived_items[0].to_dict()["itemIdentifier1"]
                else:
                    try:
                        info = str(derived_items[i].to_dict()[f])
                    except KeyError:
                        info = "Invalid Field"
                info_list.append(info)

            print_str = "\t" + derived_items[i].to_dict()['name']

            for inf in info_list:
                if inf is None:
                    inf = "None"
                print_str += " : " + inf

            # Print info to user
            try:
                click.echo(print_str)
            except KeyError:
                click.echo("Invalid Field")
                break

            found = True

    if not found:
        click.echo("invalid item name")


def get_catalog_items_by_name_help(name, field, inventory, cli):
    """
    This function prints info about catalog items given the catalog item name. Supports wildcard characters for names.

        :param name: The name of the catalog item (wildcards * and ? are supported)
        :param field: The name of the info field requested
        :param inventory: The inventory items needed
        :param cli: The necessary CliBase object
    """

    factory = cli.require_api()
    item_api = factory.getItemApi()

    catalog_items = item_api.get_catalog_items()

    found = False

    field_list = ["id", "name", "qr_id", "description", "location", "status", "model number", "function",
                  "technical system", "alternate name", "serial number"]

    fields_requested = field.split(",")

    if field == "?":
        click.echo("Valid fields:")
        click.echo("---------------")
        for f in field_list:
            click.echo(f)
        return

    # expression must be at beginning of name
    r = "^" + name.lower() + "$"

    # change SQL wildcard '?' to regex wildcard '.'
    if '?' in name:
        r = r.replace('?', '.')

    # change SQL wildcard '*' to regex wildcard '.*'
    if '*' in name:
        r = r.replace('*', '.*')

    for i in range(len(catalog_items)):

        matches = re.findall(r, (catalog_items[i]['name']).lower())

        if matches:
            info_list = []
            for f in fields_requested:
                # Get info to print
                f = f.strip(" ").lower()
                if f == "status" or f == "location" or f == "serial number":
                    info = "info not available for catalog items"
                elif f == "model number":
                    info = catalog_items[0]["itemIdentifier1"]
                elif f == "alternate name":
                    info = catalog_items[0]["itemIdentifier2"]
                elif f == "technical system":
                    info = catalog_items[0]["itemCategoryList"][0]['name']
                elif f == "function":
                    info = catalog_items[0]["itemTypeList"][0]['name']
                else:
                    try:
                        info = str(catalog_items[i][f])
                    except KeyError:
                        info = "Invalid Field"
                info_list.append(info)

            # Print info to user
            click.echo("Catalog Item:")

            print_str = "\t" + catalog_items[i]['name']

            for inf in info_list:
                if inf is None:
                    inf = "None"
                print_str += " : " + inf

            try:
                click.echo(print_str)
            except KeyError:
                click.echo("Invalid Field")

            if inventory:
                if inventory == "all":
                    click.echo("Inventory Items:")
                    get_inventory_info_by_catalog_id(catalog_items[i]['id'], field, cli)
                else:
                    click.echo("Inventory Items:")
                    get_specific_inventory_info_by_catalog_id(catalog_items[i]['id'], field, inventory, cli)

            found = True

    if not found:
        click.echo("invalid item name")


@main.command()
@click.option('--name', required=True, prompt='Item Name', help='name of the item (use wildcards * and ?)')
@click.option('--field', prompt='Field Name', help='field to be returned (use ? to get options)')
@click.option('--inventory', help='Inventory items to be included (use "all" to get all inventory for that catalog item')
@click.option('--dist', help='Change the CDB distribution (sandbox, dev, prod)')
def get_catalog_items_by_name(name, field, inventory, dist=None):
    """ Gets given field(s) and inventory item(s) for given catalog item

        \b
        * For multi-word entries enclose in ""
        * Wildcards work for catalog names and inventory names
        * Wildcard (?): Use ? as any single character
        * Wildcard (*): Use * to capture any amount of characters

        \b
        Example: get-catalog-items-by-name --name "001 - CDB*" --field id --inventory "unit: 1*"
        Example: get-catalog-items-by-name --name "001 ? CDB Test Component" --field "qr_id, description"
    """
    cli = set_cdb_dist(dist)
    get_catalog_items_by_name_help(name, field, inventory, cli)


################################################################################################
#                                                                                              #
#                                 Get location ID by name                                      #
#                                                                                              #
################################################################################################
def get_location_id_by_name_help(location_name, cli):
    """
    This function gets a locatable item id from a given location name

        :param location_name: The name of the location as a string
        :param cli: necessary CliBase object
    """

    factory = cli.require_api()
    item_api = factory.getItemApi()

    catalog = item_api.get_catalog_items()
    print(len(catalog))

    inventory = item_api.get_items_by_domain(domain_name="inventory")
    print(len(inventory))

    locations = item_api.get_items_by_domain("location")

    # expression must be at beginning of name
    r = "^" + location_name.lower() + "$"

    # change SQL wildcard '?' to regex wildcard '.'
    if '?' in location_name:
        r = r.replace('?', '.')

    # change SQL wildcard '*' to regex wildcard '.*'
    if '*' in location_name:
        r = r.replace('*', '.*')

    found = False
    click.echo("Location: ID")
    click.echo("--------------")
    for i in range(len(locations)):

        matches = re.findall(r, (locations[i].to_dict()['name']).lower())

        if matches:
            print_string = locations[i].to_dict()['name'] + ": " + str(locations[i].to_dict()['id'])
            click.echo(print_string)
            found = True
    if not found:
        click.echo("Location not found")


@main.command()
@click.option('--location-name', required=True, prompt='Location Name:', help='Location Name (use wildcards ? and *)')
@click.option('--dist', help='Change the CDB distribution (sandbox, dev, prod)')
def get_location_id_by_name(location_name, dist=None):
    """Gets the corresponding ID for a location name

        \b
        * For multi-word entries enclose entry in ""
        * Wildcard (?): Use ? as any single character
        * Wildcard (*): Use * to capture any amount of characters

        \b
        Example: get-location-id-by-name --location-name 335*
        Example: get-location-id-by-name --location-name "335?C?shelf 9"
    """
    cli = set_cdb_dist(dist)
    get_location_id_by_name_help(location_name, cli)


##############################################################################################
#                                                                                            #
#                           Add log to item given the item's ID                              #
#                                                                                            #
##############################################################################################
def set_item_log_by_id_help(item_id, log_entry, cli, effective_date=None):
    """Helper function to set a log for an item in CDB

        :param item_id: item ID of the object which the log is being written for
        :param log_entry: the log entry to be written
        :param cli: necessary CliBase object
        :param effective_date: optional date of log"""

    try:
        factory = cli.require_authenticated_api()
        itemApi = factory.getItemApi()
    except ApiException:
        click.echo("Unauthorized User/ Wrong Username or Password. Try again.")
        return
    for _id in item_id.split(','):
        try:
            log_entry_obj = LogEntryEditInformation(item_id=_id, log_entry=log_entry, effective_date=effective_date)
            itemApi.add_log_entry_to_item(log_entry_edit_information=log_entry_obj)
        except ApiException as e:
            p = r'"localizedMessage.*'
            matches = re.findall(p, e.body)
            if matches:
                error = "Error uploading log entry: " + matches[0][:-2]
                click.echo(error)
            else:
                click.echo("Error uploading log entry")
            exit(1)
    click.echo("Log updated successfully")


@main.command()
@click.option('--item-id', required=True, prompt='Item ID', help='Id of item to fetch.')
@click.option('--log-entry', required=True, prompt='Log Entry', help='Log entry text')
@click.option('--effective-date', help='Effective date of log entry. Format: "yyy-mm-dd" Default to todays date')
@click.option('--dist', help='Change the CDB distribution (sandbox, dev, prod)')
def set_item_log_by_id(item_id, log_entry, dist=None, effective_date=None):
    """Adds a log entry to the item with the given ID

        \b
        Example (default date): add-log-to-item-by-id --item-id 2160 --log-entry "Test entry"
        Example (custom date): add-log-to-item-by-id --item-id 2160 --log-entry "Test entry" --effective-date 2020-06-22
        Example (multiple IDs): add-log-to-item-by-id --item-id "2160, 3604, 3605" --log-entry "Test entry"
    """
    cli = set_cdb_dist(dist)
    set_item_log_by_id_help(item_id, log_entry, cli, effective_date)


##############################################################################################
#                                                                                            #
#                              Update item status given item ID                              #
#                                                                                            #
##############################################################################################
def set_item_status_by_id_help(item_id, status, cli):
    """
    This function updates the status of a given item.

        :param item_id: ID of the inventory item. Multiple IDs can be given as a comma separated string
        :param status: new status of the item. Only statuses from "statusDict.json" are allowed as parameters
        :param cli: necessary CliBase object
    """

    factory = cli.require_authenticated_api()
    item_api = factory.getItemApi()
    prop_api = factory.getPropertyApi()

    status_prop = prop_api.get_inventory_status_property_type()

    status_list = [status.to_dict()['value'] for status in status_prop.sorted_allowed_property_value_list]

    if status == "?":
        click.echo("Status Options:")
        click.echo("----------------")
        for stat in status_list:
            click.echo(stat)
        return

    try:
        item_status = ItemStatusBasicObject(status=status)

        for _id in item_id.split(','):
            try:
                result = item_api.update_item_status(item_id=int(_id.strip(' ')), item_status_basic_object=item_status)
                log = "Status updated to " + status + " by CDB CLI"
                set_item_log_by_id_help(_id, log, cli, effective_date=None)
            except ApiException as e:
                p = r'"localizedMessage.*'
                matches = re.findall(p, e.body)
                if matches:
                    error = "Error updating status: " + matches[0][:-2]
                    click.echo(error)
                else:
                    click.echo("Error updating status")
                if status not in status_list:
                    click.echo("Please enter valid status from status list:")
                    for stat in status_list:
                        click.echo(stat)
                exit(1)

    except KeyError:
        click.echo("Error: invalid status entered")


@main.command()
@click.option('--item-id', required=True, prompt='Item ID', help='Single item ID or list of item IDs', type=str)
@click.option('--status', required=True, prompt='Item Status (? for options)', help='New Status of Item')
@click.option('--dist', help='Change the CDB distribution (sandbox, dev, prod)')
def set_item_status_by_id(item_id, status, dist=None):
    """Updates item status of item with the given ID and updates item log

        \b
        * For multi-word entries, enclose entry in ""
        * For list of IDs: enter as "ID1, ID2, ID3, ..."

        \b
        Example (single item): set-item-status-by-id --item-id 2160 --status "Ready For Use"
        Example (multi-item): set-item-status-by-id --item-id "2160, 3604, 3605" --status Spare
    """
    cli = set_cdb_dist(dist)
    set_item_status_by_id_help(item_id, status, cli)


################################################################################################
#                                                                                              #
#                                 Set new location of item                                     #
#                                                                                              #
################################################################################################
def set_location_by_id_help(item_id, location_id, cli):
    """
    This function sets a new location for a given inventory item

        :param item_id: The ID of the inventory item
        :param location_id: The ID of the location item
        :param cli: necessary CliBase object
    """

    factory = cli.require_authenticated_api()
    item_api = factory.getItemApi()

    _ids = item_id.split(",")
    for _id in _ids:
        location = SimpleLocationInformation(locatable_item_id=_id.strip(" "), location_item_id=location_id)
        try:
            item_api.update_item_location(simple_location_information=location)
        except ApiException as e:
            p = r'"localizedMessage.*'
            matches = re.findall(p, e.body)
            if matches:
                error = "Error updating location: " + matches[0][:-2]
                click.echo(error)
            exit(1)

    log = "Location updated by CDB CLI"
    set_item_log_by_id_help(item_id, log, cli, effective_date=None)


@main.command()
@click.option('--item-id', required=True, prompt='Item ID',
              help='ID of inventory item - single id or comma separated list')
@click.option('--location-id', required=True, prompt='Item Location ID', help='New location ID for item')
@click.option('--dist', help='Change the CDB distribution (sandbox, dev, prod)')
def set_location_by_id(item_id, location_id, dist=None):
    """Set new location for single or multiple items using location ID

        \b
        * For list of IDs: enter as "ID1, ID2, ID3, ..."

        \b
        Example (single item): set-location-by-id --item-id 2160 --location-id 6565
        Example (multi-item): set-location-by-id --item-id "2160, 3604, 3605" --location-id 6565"""
    cli = set_cdb_dist(dist)
    set_location_by_id_help(item_id, location_id, cli)


##############################################################################################
#                                                                                            #
#                             Set the QR ID of an inventory item                             #
#                                                                                            #
##############################################################################################
def set_qr_id_by_id_help(item_id, qrid, cli):
    """
    This function sets a new QR ID for a given item

        :param item_id: The ID of the item
        :param qrid: The new desired QR ID of the item
        :param cli: necessary CliBase object
    """

    factory = cli.require_authenticated_api()
    item_api = factory.getItemApi()

    try:
        item = item_api.get_item_by_id(item_id)
        item.qr_id = qrid
        item_api.update_item_details(item=item)
    except ApiException as e:
        p = r'"localizedMessage.*'
        matches = re.findall(p, e.body)
        if matches:
            error = "Error setting QR ID: " + matches[0][:-2]
            click.echo(error)
        else:
            click.echo("Error setting QR ID")
        exit(1)


@main.command()
@click.option('--item-id', required=True, prompt='Item ID', help='ID of the item (NOT the qr ID)')
@click.option('--qr-id', required=True, prompt='QR ID', help='new QR ID', type=int)
@click.option('--dist', help='Change the CDB distribution (sandbox, dev, prod)')
def set_qr_id_by_id(item_id, qr_id, dist=None):
    """ Sets new qr ID for given item ID

        \b
        Example: set-qr-id-by-id --item-id 2106 --qr-id 1359
    """
    cli = set_cdb_dist(dist)
    set_qr_id_by_id_help(item_id, qr_id, cli)


if __name__ == "__main__":
    main()



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from cdbApi.api.authentication_api import AuthenticationApi
from cdbApi.api.downloads_api import DownloadsApi
from cdbApi.api.item_api import ItemApi
from cdbApi.api.log_api import LogApi
from cdbApi.api.property_type_api import PropertyTypeApi
from cdbApi.api.property_value_api import PropertyValueApi
from cdbApi.api.test_api import TestApi
from cdbApi.api.users_api import UsersApi
from cdbApi.api.cable_catalog_items_api import CableCatalogItemsApi
from cdbApi.api.cable_design_items_api import CableDesignItemsApi
from cdbApi.api.domain_api import DomainApi
from cdbApi.api.location_items_api import LocationItemsApi
from cdbApi.api.machine_design_items_api import MachineDesignItemsApi
from cdbApi.api.sources_api import SourcesApi



# coding: utf-8

"""
    Component Database API

    The API that provides access to Component Database data.  # noqa: E501

    The version of the OpenAPI document: 3.8.0
    Contact: djarosz@anl.gov
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from cdbApi.configuration import Configuration


class ItemPermissions(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'item_id': 'int',
        'owner_user': 'UserInfo',
        'owner_group': 'UserGroup',
        'group_writeable': 'bool'
    }

    attribute_map = {
        'item_id': 'itemId',
        'owner_user': 'ownerUser',
        'owner_group': 'ownerGroup',
        'group_writeable': 'groupWriteable'
    }

    def __init__(self, item_id=None, owner_user=None, owner_group=None, group_writeable=None, local_vars_configuration=None):  # noqa: E501
        """ItemPermissions - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._item_id = None
        self._owner_user = None
        self._owner_group = None
        self._group_writeable = None
        self.discriminator = None

        if item_id is not None:
            self.item_id = item_id
        if owner_user is not None:
            self.owner_user = owner_user
        if owner_group is not None:
            self.owner_group = owner_group
        if group_writeable is not None:
            self.group_writeable = group_writeable

    @property
    def item_id(self):
        """Gets the item_id of this ItemPermissions.  # noqa: E501


        :return: The item_id of this ItemPermissions.  # noqa: E501
        :rtype: int
        """
        return self._item_id

    @item_id.setter
    def item_id(self, item_id):
        """Sets the item_id of this ItemPermissions.


        :param item_id: The item_id of this ItemPermissions.  # noqa: E501
        :type: int
        """

        self._item_id = item_id

    @property
    def owner_user(self):
        """Gets the owner_user of this ItemPermissions.  # noqa: E501


        :return: The owner_user of this ItemPermissions.  # noqa: E501
        :rtype: UserInfo
        """
        return self._owner_user

    @owner_user.setter
    def owner_user(self, owner_user):
        """Sets the owner_user of this ItemPermissions.


        :param owner_user: The owner_user of this ItemPermissions.  # noqa: E501
        :type: UserInfo
        """

        self._owner_user = owner_user

    @property
    def owner_group(self):
        """Gets the owner_group of this ItemPermissions.  # noqa: E501


        :return: The owner_group of this ItemPermissions.  # noqa: E501
        :rtype: UserGroup
        """
        return self._owner_group

    @owner_group.setter
    def owner_group(self, owner_group):
        """Sets the owner_group of this ItemPermissions.


        :param owner_group: The owner_group of this ItemPermissions.  # noqa: E501
        :type: UserGroup
        """

        self._owner_group = owner_group

    @property
    def group_writeable(self):
        """Gets the group_writeable of this ItemPermissions.  # noqa: E501


        :return: The group_writeable of this ItemPermissions.  # noqa: E501
        :rtype: bool
        """
        return self._group_writeable

    @group_writeable.setter
    def group_writeable(self, group_writeable):
        """Sets the group_writeable of this ItemPermissions.


        :param group_writeable: The group_writeable of this ItemPermissions.  # noqa: E501
        :type: bool
        """

        self._group_writeable = group_writeable

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ItemPermissions):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ItemPermissions):
            return True

        return self.to_dict() != other.to_dict()



import os

from click._compat import raw_input

from getpass import getpass

from CdbApiFactory import CdbApiFactory
from cdbApi import ApiException
from cdbCli.common.utility.configurationManager import ConfigurationManager


class CliBase:

    api_factory = None

    def __init__(self):
        self.config = ConfigurationManager.get_instance()

    def _get_api_factory(self):
        if CliBase.api_factory is None:
            portal_URL = self.config.get_portal_address()
            CliBase.api_factory = CdbApiFactory(portal_URL)

        return CliBase.api_factory

    def get_session_token(self):
        session_file_path = self.config.get_session_file_path()
        if os.path.exists(session_file_path):
            session_file = open(session_file_path, 'r')
            return session_file.read()
        return None

    def set_session_token(self, session_token):
        session_file_path = self.config.get_session_file_path()
        directory = os.path.dirname(session_file_path)
        if not os.path.exists(directory):
            os.makedirs(directory)

        session_file = open(session_file_path, 'w')
        session_file.write(session_token)

    def require_api(self):
        apiFactory = self._get_api_factory()
        return apiFactory

    def require_authenticated_api(self):
        factory = self.require_api()
        factory.getItemApi()

        try:
            token = self.get_session_token()
            if token is not None:
                factory.setAuthenticateToken(token)
            factory.testAuthenticated()
        except ApiException as ex:
            # Need to prompt for credentials
            print("The command requires authentication.")
            username = raw_input("Username: ")
            password = getpass("Password: ")
            try:
                factory.authenticateUser(username, password)
                token = factory.getAuthenticateToken()
                self.set_session_token(token)
            except ApiException as ex:
                raise ex
                print("Authentication failed.")

        return factory

    @staticmethod
    def print_cdb_obj(cdb_object):
        cdb_object = str(cdb_object) + '\n'
        print(cdb_object)

    # TODO Add a print list cdb of cdb object



#!/usr/bin/env python3

import click

from cdbCli.common.cli.cliBase import CliBase


@click.command()
@click.option('--item-id', prompt='Item ID',
              help='Id of item to fetch.')
def get_item_by_id(item_id):
    cli = CliBase()
    factory = cli.require_api()
    itemApi = factory.getItemApi()

    item = itemApi.get_item_by_id(item_id)

    cli.print_cdb_obj(item)


if __name__ == "__main__":
    get_item_by_id()



#!/usr/bin/env python3

import click

from cdbApi import LogEntryEditInformation
from cdbCli.common.cli.cliBase import CliBase


@click.command()
@click.option('--item-id', prompt='Item ID',
              help='Id of item to fetch.')
@click.option('--log-entry', prompt='Log Entry',
              help='Log entry text')
@click.option('--effective-date', help='Effective date of log entry')
def add_log_to_item_by_id(item_id, log_entry, effective_date=None):
    cli = CliBase()
    factory = cli.require_authenticated_api()
    itemApi = factory.getItemApi()

    log_entry_obj = LogEntryEditInformation(item_id=item_id, log_entry=log_entry, effective_date=effective_date)
    log = itemApi.add_log_entry_to_item(log_entry_edit_information=log_entry_obj)

    cli.print_cdb_obj(log)


if __name__ == "__main__":
    add_log_to_item_by_id()



# coding: utf-8

"""
    Component Database API

    The API that provides access to Component Database data.  # noqa: E501

    The version of the OpenAPI document: 3.8.0
    Contact: djarosz@anl.gov
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from cdbApi.configuration import Configuration


class ItemElement(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'int',
        'name': 'str',
        'is_required': 'bool',
        'description': 'str',
        'sort_order': 'float',
        'list_list': 'list[ListTbl]',
        'property_value_list': 'list[PropertyValue]',
        'log_list': 'list[Log]',
        'derived_from_item_element_list': 'list[ItemElement]',
        'derived_from_item_element': 'ItemElement',
        'item_element_history_list': 'list[ItemElementHistory]',
        'row_style': 'str',
        'parent_item_id': 'int',
        'contained_item1_id': 'int',
        'contained_item2_id': 'int'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'is_required': 'isRequired',
        'description': 'description',
        'sort_order': 'sortOrder',
        'list_list': 'listList',
        'property_value_list': 'propertyValueList',
        'log_list': 'logList',
        'derived_from_item_element_list': 'derivedFromItemElementList',
        'derived_from_item_element': 'derivedFromItemElement',
        'item_element_history_list': 'itemElementHistoryList',
        'row_style': 'rowStyle',
        'parent_item_id': 'parentItemId',
        'contained_item1_id': 'containedItem1Id',
        'contained_item2_id': 'containedItem2Id'
    }

    def __init__(self, id=None, name=None, is_required=None, description=None, sort_order=None, list_list=None, property_value_list=None, log_list=None, derived_from_item_element_list=None, derived_from_item_element=None, item_element_history_list=None, row_style=None, parent_item_id=None, contained_item1_id=None, contained_item2_id=None, local_vars_configuration=None):  # noqa: E501
        """ItemElement - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._is_required = None
        self._description = None
        self._sort_order = None
        self._list_list = None
        self._property_value_list = None
        self._log_list = None
        self._derived_from_item_element_list = None
        self._derived_from_item_element = None
        self._item_element_history_list = None
        self._row_style = None
        self._parent_item_id = None
        self._contained_item1_id = None
        self._contained_item2_id = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if is_required is not None:
            self.is_required = is_required
        if description is not None:
            self.description = description
        if sort_order is not None:
            self.sort_order = sort_order
        if list_list is not None:
            self.list_list = list_list
        if property_value_list is not None:
            self.property_value_list = property_value_list
        if log_list is not None:
            self.log_list = log_list
        if derived_from_item_element_list is not None:
            self.derived_from_item_element_list = derived_from_item_element_list
        if derived_from_item_element is not None:
            self.derived_from_item_element = derived_from_item_element
        if item_element_history_list is not None:
            self.item_element_history_list = item_element_history_list
        if row_style is not None:
            self.row_style = row_style
        if parent_item_id is not None:
            self.parent_item_id = parent_item_id
        if contained_item1_id is not None:
            self.contained_item1_id = contained_item1_id
        if contained_item2_id is not None:
            self.contained_item2_id = contained_item2_id

    @property
    def id(self):
        """Gets the id of this ItemElement.  # noqa: E501


        :return: The id of this ItemElement.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ItemElement.


        :param id: The id of this ItemElement.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this ItemElement.  # noqa: E501


        :return: The name of this ItemElement.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ItemElement.


        :param name: The name of this ItemElement.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                name is not None and len(name) > 64):
            raise ValueError("Invalid value for `name`, length must be less than or equal to `64`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                name is not None and len(name) < 0):
            raise ValueError("Invalid value for `name`, length must be greater than or equal to `0`")  # noqa: E501

        self._name = name

    @property
    def is_required(self):
        """Gets the is_required of this ItemElement.  # noqa: E501


        :return: The is_required of this ItemElement.  # noqa: E501
        :rtype: bool
        """
        return self._is_required

    @is_required.setter
    def is_required(self, is_required):
        """Sets the is_required of this ItemElement.


        :param is_required: The is_required of this ItemElement.  # noqa: E501
        :type: bool
        """

        self._is_required = is_required

    @property
    def description(self):
        """Gets the description of this ItemElement.  # noqa: E501


        :return: The description of this ItemElement.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ItemElement.


        :param description: The description of this ItemElement.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                description is not None and len(description) > 256):
            raise ValueError("Invalid value for `description`, length must be less than or equal to `256`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                description is not None and len(description) < 0):
            raise ValueError("Invalid value for `description`, length must be greater than or equal to `0`")  # noqa: E501

        self._description = description

    @property
    def sort_order(self):
        """Gets the sort_order of this ItemElement.  # noqa: E501


        :return: The sort_order of this ItemElement.  # noqa: E501
        :rtype: float
        """
        return self._sort_order

    @sort_order.setter
    def sort_order(self, sort_order):
        """Sets the sort_order of this ItemElement.


        :param sort_order: The sort_order of this ItemElement.  # noqa: E501
        :type: float
        """

        self._sort_order = sort_order

    @property
    def list_list(self):
        """Gets the list_list of this ItemElement.  # noqa: E501


        :return: The list_list of this ItemElement.  # noqa: E501
        :rtype: list[ListTbl]
        """
        return self._list_list

    @list_list.setter
    def list_list(self, list_list):
        """Sets the list_list of this ItemElement.


        :param list_list: The list_list of this ItemElement.  # noqa: E501
        :type: list[ListTbl]
        """

        self._list_list = list_list

    @property
    def property_value_list(self):
        """Gets the property_value_list of this ItemElement.  # noqa: E501


        :return: The property_value_list of this ItemElement.  # noqa: E501
        :rtype: list[PropertyValue]
        """
        return self._property_value_list

    @property_value_list.setter
    def property_value_list(self, property_value_list):
        """Sets the property_value_list of this ItemElement.


        :param property_value_list: The property_value_list of this ItemElement.  # noqa: E501
        :type: list[PropertyValue]
        """

        self._property_value_list = property_value_list

    @property
    def log_list(self):
        """Gets the log_list of this ItemElement.  # noqa: E501


        :return: The log_list of this ItemElement.  # noqa: E501
        :rtype: list[Log]
        """
        return self._log_list

    @log_list.setter
    def log_list(self, log_list):
        """Sets the log_list of this ItemElement.


        :param log_list: The log_list of this ItemElement.  # noqa: E501
        :type: list[Log]
        """

        self._log_list = log_list

    @property
    def derived_from_item_element_list(self):
        """Gets the derived_from_item_element_list of this ItemElement.  # noqa: E501


        :return: The derived_from_item_element_list of this ItemElement.  # noqa: E501
        :rtype: list[ItemElement]
        """
        return self._derived_from_item_element_list

    @derived_from_item_element_list.setter
    def derived_from_item_element_list(self, derived_from_item_element_list):
        """Sets the derived_from_item_element_list of this ItemElement.


        :param derived_from_item_element_list: The derived_from_item_element_list of this ItemElement.  # noqa: E501
        :type: list[ItemElement]
        """

        self._derived_from_item_element_list = derived_from_item_element_list

    @property
    def derived_from_item_element(self):
        """Gets the derived_from_item_element of this ItemElement.  # noqa: E501


        :return: The derived_from_item_element of this ItemElement.  # noqa: E501
        :rtype: ItemElement
        """
        return self._derived_from_item_element

    @derived_from_item_element.setter
    def derived_from_item_element(self, derived_from_item_element):
        """Sets the derived_from_item_element of this ItemElement.


        :param derived_from_item_element: The derived_from_item_element of this ItemElement.  # noqa: E501
        :type: ItemElement
        """

        self._derived_from_item_element = derived_from_item_element

    @property
    def item_element_history_list(self):
        """Gets the item_element_history_list of this ItemElement.  # noqa: E501


        :return: The item_element_history_list of this ItemElement.  # noqa: E501
        :rtype: list[ItemElementHistory]
        """
        return self._item_element_history_list

    @item_element_history_list.setter
    def item_element_history_list(self, item_element_history_list):
        """Sets the item_element_history_list of this ItemElement.


        :param item_element_history_list: The item_element_history_list of this ItemElement.  # noqa: E501
        :type: list[ItemElementHistory]
        """

        self._item_element_history_list = item_element_history_list

    @property
    def row_style(self):
        """Gets the row_style of this ItemElement.  # noqa: E501


        :return: The row_style of this ItemElement.  # noqa: E501
        :rtype: str
        """
        return self._row_style

    @row_style.setter
    def row_style(self, row_style):
        """Sets the row_style of this ItemElement.


        :param row_style: The row_style of this ItemElement.  # noqa: E501
        :type: str
        """

        self._row_style = row_style

    @property
    def parent_item_id(self):
        """Gets the parent_item_id of this ItemElement.  # noqa: E501


        :return: The parent_item_id of this ItemElement.  # noqa: E501
        :rtype: int
        """
        return self._parent_item_id

    @parent_item_id.setter
    def parent_item_id(self, parent_item_id):
        """Sets the parent_item_id of this ItemElement.


        :param parent_item_id: The parent_item_id of this ItemElement.  # noqa: E501
        :type: int
        """

        self._parent_item_id = parent_item_id

    @property
    def contained_item1_id(self):
        """Gets the contained_item1_id of this ItemElement.  # noqa: E501


        :return: The contained_item1_id of this ItemElement.  # noqa: E501
        :rtype: int
        """
        return self._contained_item1_id

    @contained_item1_id.setter
    def contained_item1_id(self, contained_item1_id):
        """Sets the contained_item1_id of this ItemElement.


        :param contained_item1_id: The contained_item1_id of this ItemElement.  # noqa: E501
        :type: int
        """

        self._contained_item1_id = contained_item1_id

    @property
    def contained_item2_id(self):
        """Gets the contained_item2_id of this ItemElement.  # noqa: E501


        :return: The contained_item2_id of this ItemElement.  # noqa: E501
        :rtype: int
        """
        return self._contained_item2_id

    @contained_item2_id.setter
    def contained_item2_id(self, contained_item2_id):
        """Sets the contained_item2_id of this ItemElement.


        :param contained_item2_id: The contained_item2_id of this ItemElement.  # noqa: E501
        :type: int
        """

        self._contained_item2_id = contained_item2_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ItemElement):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ItemElement):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Component Database API

    The API that provides access to Component Database data.  # noqa: E501

    The version of the OpenAPI document: 3.8.0
    Contact: djarosz@anl.gov
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from cdbApi.configuration import Configuration


class ConciseItemOptions(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'include_derived_from_item_info': 'bool',
        'include_item_project_id_list': 'bool',
        'include_item_type_id_list': 'bool',
        'include_item_category_id_list': 'bool',
        'include_primary_image_for_item': 'bool'
    }

    attribute_map = {
        'include_derived_from_item_info': 'includeDerivedFromItemInfo',
        'include_item_project_id_list': 'includeItemProjectIdList',
        'include_item_type_id_list': 'includeItemTypeIdList',
        'include_item_category_id_list': 'includeItemCategoryIdList',
        'include_primary_image_for_item': 'includePrimaryImageForItem'
    }

    def __init__(self, include_derived_from_item_info=None, include_item_project_id_list=None, include_item_type_id_list=None, include_item_category_id_list=None, include_primary_image_for_item=None, local_vars_configuration=None):  # noqa: E501
        """ConciseItemOptions - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._include_derived_from_item_info = None
        self._include_item_project_id_list = None
        self._include_item_type_id_list = None
        self._include_item_category_id_list = None
        self._include_primary_image_for_item = None
        self.discriminator = None

        if include_derived_from_item_info is not None:
            self.include_derived_from_item_info = include_derived_from_item_info
        if include_item_project_id_list is not None:
            self.include_item_project_id_list = include_item_project_id_list
        if include_item_type_id_list is not None:
            self.include_item_type_id_list = include_item_type_id_list
        if include_item_category_id_list is not None:
            self.include_item_category_id_list = include_item_category_id_list
        if include_primary_image_for_item is not None:
            self.include_primary_image_for_item = include_primary_image_for_item

    @property
    def include_derived_from_item_info(self):
        """Gets the include_derived_from_item_info of this ConciseItemOptions.  # noqa: E501


        :return: The include_derived_from_item_info of this ConciseItemOptions.  # noqa: E501
        :rtype: bool
        """
        return self._include_derived_from_item_info

    @include_derived_from_item_info.setter
    def include_derived_from_item_info(self, include_derived_from_item_info):
        """Sets the include_derived_from_item_info of this ConciseItemOptions.


        :param include_derived_from_item_info: The include_derived_from_item_info of this ConciseItemOptions.  # noqa: E501
        :type: bool
        """

        self._include_derived_from_item_info = include_derived_from_item_info

    @property
    def include_item_project_id_list(self):
        """Gets the include_item_project_id_list of this ConciseItemOptions.  # noqa: E501


        :return: The include_item_project_id_list of this ConciseItemOptions.  # noqa: E501
        :rtype: bool
        """
        return self._include_item_project_id_list

    @include_item_project_id_list.setter
    def include_item_project_id_list(self, include_item_project_id_list):
        """Sets the include_item_project_id_list of this ConciseItemOptions.


        :param include_item_project_id_list: The include_item_project_id_list of this ConciseItemOptions.  # noqa: E501
        :type: bool
        """

        self._include_item_project_id_list = include_item_project_id_list

    @property
    def include_item_type_id_list(self):
        """Gets the include_item_type_id_list of this ConciseItemOptions.  # noqa: E501


        :return: The include_item_type_id_list of this ConciseItemOptions.  # noqa: E501
        :rtype: bool
        """
        return self._include_item_type_id_list

    @include_item_type_id_list.setter
    def include_item_type_id_list(self, include_item_type_id_list):
        """Sets the include_item_type_id_list of this ConciseItemOptions.


        :param include_item_type_id_list: The include_item_type_id_list of this ConciseItemOptions.  # noqa: E501
        :type: bool
        """

        self._include_item_type_id_list = include_item_type_id_list

    @property
    def include_item_category_id_list(self):
        """Gets the include_item_category_id_list of this ConciseItemOptions.  # noqa: E501


        :return: The include_item_category_id_list of this ConciseItemOptions.  # noqa: E501
        :rtype: bool
        """
        return self._include_item_category_id_list

    @include_item_category_id_list.setter
    def include_item_category_id_list(self, include_item_category_id_list):
        """Sets the include_item_category_id_list of this ConciseItemOptions.


        :param include_item_category_id_list: The include_item_category_id_list of this ConciseItemOptions.  # noqa: E501
        :type: bool
        """

        self._include_item_category_id_list = include_item_category_id_list

    @property
    def include_primary_image_for_item(self):
        """Gets the include_primary_image_for_item of this ConciseItemOptions.  # noqa: E501


        :return: The include_primary_image_for_item of this ConciseItemOptions.  # noqa: E501
        :rtype: bool
        """
        return self._include_primary_image_for_item

    @include_primary_image_for_item.setter
    def include_primary_image_for_item(self, include_primary_image_for_item):
        """Sets the include_primary_image_for_item of this ConciseItemOptions.


        :param include_primary_image_for_item: The include_primary_image_for_item of this ConciseItemOptions.  # noqa: E501
        :type: bool
        """

        self._include_primary_image_for_item = include_primary_image_for_item

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConciseItemOptions):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ConciseItemOptions):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Component Database API

    The API that provides access to Component Database data.  # noqa: E501

    The version of the OpenAPI document: 3.8.0
    Contact: djarosz@anl.gov
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from cdbApi.configuration import Configuration


class PropertyType(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'int',
        'name': 'str',
        'description': 'str',
        'prompt_description': 'str',
        'default_value': 'str',
        'default_units': 'str',
        'is_user_writeable': 'bool',
        'is_dynamic': 'bool',
        'is_internal': 'bool',
        'is_active': 'bool',
        'is_metadata_dynamic': 'bool',
        'sorted_allowed_property_value_list': 'list[AllowedPropertyValue]'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'description': 'description',
        'prompt_description': 'promptDescription',
        'default_value': 'defaultValue',
        'default_units': 'defaultUnits',
        'is_user_writeable': 'isUserWriteable',
        'is_dynamic': 'isDynamic',
        'is_internal': 'isInternal',
        'is_active': 'isActive',
        'is_metadata_dynamic': 'isMetadataDynamic',
        'sorted_allowed_property_value_list': 'sortedAllowedPropertyValueList'
    }

    def __init__(self, id=None, name=None, description=None, prompt_description=None, default_value=None, default_units=None, is_user_writeable=None, is_dynamic=None, is_internal=None, is_active=None, is_metadata_dynamic=None, sorted_allowed_property_value_list=None, local_vars_configuration=None):  # noqa: E501
        """PropertyType - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._description = None
        self._prompt_description = None
        self._default_value = None
        self._default_units = None
        self._is_user_writeable = None
        self._is_dynamic = None
        self._is_internal = None
        self._is_active = None
        self._is_metadata_dynamic = None
        self._sorted_allowed_property_value_list = None
        self.discriminator = None

        if id is not None:
            self.id = id
        self.name = name
        if description is not None:
            self.description = description
        if prompt_description is not None:
            self.prompt_description = prompt_description
        if default_value is not None:
            self.default_value = default_value
        if default_units is not None:
            self.default_units = default_units
        if is_user_writeable is not None:
            self.is_user_writeable = is_user_writeable
        if is_dynamic is not None:
            self.is_dynamic = is_dynamic
        if is_internal is not None:
            self.is_internal = is_internal
        if is_active is not None:
            self.is_active = is_active
        if is_metadata_dynamic is not None:
            self.is_metadata_dynamic = is_metadata_dynamic
        if sorted_allowed_property_value_list is not None:
            self.sorted_allowed_property_value_list = sorted_allowed_property_value_list

    @property
    def id(self):
        """Gets the id of this PropertyType.  # noqa: E501


        :return: The id of this PropertyType.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this PropertyType.


        :param id: The id of this PropertyType.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this PropertyType.  # noqa: E501


        :return: The name of this PropertyType.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this PropertyType.


        :param name: The name of this PropertyType.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                name is not None and len(name) > 64):
            raise ValueError("Invalid value for `name`, length must be less than or equal to `64`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                name is not None and len(name) < 1):
            raise ValueError("Invalid value for `name`, length must be greater than or equal to `1`")  # noqa: E501

        self._name = name

    @property
    def description(self):
        """Gets the description of this PropertyType.  # noqa: E501


        :return: The description of this PropertyType.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this PropertyType.


        :param description: The description of this PropertyType.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                description is not None and len(description) > 256):
            raise ValueError("Invalid value for `description`, length must be less than or equal to `256`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                description is not None and len(description) < 0):
            raise ValueError("Invalid value for `description`, length must be greater than or equal to `0`")  # noqa: E501

        self._description = description

    @property
    def prompt_description(self):
        """Gets the prompt_description of this PropertyType.  # noqa: E501


        :return: The prompt_description of this PropertyType.  # noqa: E501
        :rtype: str
        """
        return self._prompt_description

    @prompt_description.setter
    def prompt_description(self, prompt_description):
        """Sets the prompt_description of this PropertyType.


        :param prompt_description: The prompt_description of this PropertyType.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                prompt_description is not None and len(prompt_description) > 256):
            raise ValueError("Invalid value for `prompt_description`, length must be less than or equal to `256`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                prompt_description is not None and len(prompt_description) < 0):
            raise ValueError("Invalid value for `prompt_description`, length must be greater than or equal to `0`")  # noqa: E501

        self._prompt_description = prompt_description

    @property
    def default_value(self):
        """Gets the default_value of this PropertyType.  # noqa: E501


        :return: The default_value of this PropertyType.  # noqa: E501
        :rtype: str
        """
        return self._default_value

    @default_value.setter
    def default_value(self, default_value):
        """Sets the default_value of this PropertyType.


        :param default_value: The default_value of this PropertyType.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                default_value is not None and len(default_value) > 64):
            raise ValueError("Invalid value for `default_value`, length must be less than or equal to `64`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                default_value is not None and len(default_value) < 0):
            raise ValueError("Invalid value for `default_value`, length must be greater than or equal to `0`")  # noqa: E501

        self._default_value = default_value

    @property
    def default_units(self):
        """Gets the default_units of this PropertyType.  # noqa: E501


        :return: The default_units of this PropertyType.  # noqa: E501
        :rtype: str
        """
        return self._default_units

    @default_units.setter
    def default_units(self, default_units):
        """Sets the default_units of this PropertyType.


        :param default_units: The default_units of this PropertyType.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                default_units is not None and len(default_units) > 16):
            raise ValueError("Invalid value for `default_units`, length must be less than or equal to `16`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                default_units is not None and len(default_units) < 0):
            raise ValueError("Invalid value for `default_units`, length must be greater than or equal to `0`")  # noqa: E501

        self._default_units = default_units

    @property
    def is_user_writeable(self):
        """Gets the is_user_writeable of this PropertyType.  # noqa: E501


        :return: The is_user_writeable of this PropertyType.  # noqa: E501
        :rtype: bool
        """
        return self._is_user_writeable

    @is_user_writeable.setter
    def is_user_writeable(self, is_user_writeable):
        """Sets the is_user_writeable of this PropertyType.


        :param is_user_writeable: The is_user_writeable of this PropertyType.  # noqa: E501
        :type: bool
        """

        self._is_user_writeable = is_user_writeable

    @property
    def is_dynamic(self):
        """Gets the is_dynamic of this PropertyType.  # noqa: E501


        :return: The is_dynamic of this PropertyType.  # noqa: E501
        :rtype: bool
        """
        return self._is_dynamic

    @is_dynamic.setter
    def is_dynamic(self, is_dynamic):
        """Sets the is_dynamic of this PropertyType.


        :param is_dynamic: The is_dynamic of this PropertyType.  # noqa: E501
        :type: bool
        """

        self._is_dynamic = is_dynamic

    @property
    def is_internal(self):
        """Gets the is_internal of this PropertyType.  # noqa: E501


        :return: The is_internal of this PropertyType.  # noqa: E501
        :rtype: bool
        """
        return self._is_internal

    @is_internal.setter
    def is_internal(self, is_internal):
        """Sets the is_internal of this PropertyType.


        :param is_internal: The is_internal of this PropertyType.  # noqa: E501
        :type: bool
        """

        self._is_internal = is_internal

    @property
    def is_active(self):
        """Gets the is_active of this PropertyType.  # noqa: E501


        :return: The is_active of this PropertyType.  # noqa: E501
        :rtype: bool
        """
        return self._is_active

    @is_active.setter
    def is_active(self, is_active):
        """Sets the is_active of this PropertyType.


        :param is_active: The is_active of this PropertyType.  # noqa: E501
        :type: bool
        """

        self._is_active = is_active

    @property
    def is_metadata_dynamic(self):
        """Gets the is_metadata_dynamic of this PropertyType.  # noqa: E501


        :return: The is_metadata_dynamic of this PropertyType.  # noqa: E501
        :rtype: bool
        """
        return self._is_metadata_dynamic

    @is_metadata_dynamic.setter
    def is_metadata_dynamic(self, is_metadata_dynamic):
        """Sets the is_metadata_dynamic of this PropertyType.


        :param is_metadata_dynamic: The is_metadata_dynamic of this PropertyType.  # noqa: E501
        :type: bool
        """

        self._is_metadata_dynamic = is_metadata_dynamic

    @property
    def sorted_allowed_property_value_list(self):
        """Gets the sorted_allowed_property_value_list of this PropertyType.  # noqa: E501


        :return: The sorted_allowed_property_value_list of this PropertyType.  # noqa: E501
        :rtype: list[AllowedPropertyValue]
        """
        return self._sorted_allowed_property_value_list

    @sorted_allowed_property_value_list.setter
    def sorted_allowed_property_value_list(self, sorted_allowed_property_value_list):
        """Sets the sorted_allowed_property_value_list of this PropertyType.


        :param sorted_allowed_property_value_list: The sorted_allowed_property_value_list of this PropertyType.  # noqa: E501
        :type: list[AllowedPropertyValue]
        """

        self._sorted_allowed_property_value_list = sorted_allowed_property_value_list

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PropertyType):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, PropertyType):
            return True

        return self.to_dict() != other.to_dict()
